##
# Copyright (c) 2021, Xilinx Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
##
from flask import jsonify
from app import logger

exception_hash = {
"ArithmeticError":{"message":"Invalid Logical Expression","error_code":500},
"AssertionError":{"message":"Internal Server Error","error_code":500},
"AttributeError":{"message":"Attribute Missing","error_code":500},
"BufferError":{"message":"Internal Buffer Error","error_code":500},
"EOFError":{"message":"Internal Server Error","error_code":500},
"ModuleNotFoundError":{"message":"Module not found","error_code":500},
"IndexError":{"message":"Key Not Found","error_code":500},
"KeyError":{"message":"Key Not Found","error_code":500},
"MemoryError":{"message":"Memory Exceed","error_code":500},
"NameError":{"message":"Internal Server Error","error_code":500},
"OSError":{"message":"Internal Server Error","error_code":500},
"FileExistsError":{"message":"Internal Server Error","error_code":500},
"FileNotFoundError":{"message":"File Not Found","error_code":500},
"InterruptedError":{"message":"Keyboard interrupt","error_code":500},
"IsADirectoryError":{"message":"Keyboard interrupt","error_code":500},
"NotADirectoryError":{"message":"Keyboard interrupt","error_code":500},
"PermissionError":{"message":"Permission Required","error_code":400},
"ProcessLookupError":{"message":"Keyboard interrupt","error_code":500},
"TimeoutError":{"message":"Time Out","error_code":400},
"ReferenceError":{"message":"Refference Not Found","error_code":500},
"NotImplementedError":{"message":"Method Not Found","error_code":500},
"RecursionError":{"message":"Internal Server Error","error_code":500},
"StopIteration":{"message":"Internal Server Error","error_code":500},
"StopAsyncIteration":{"message":"Internal Server Error","error_code":500},
"SyntaxError":{"message":"Syntax Error","error_code":1},
"SystemError":{"message":"Internal Server Error","error_code":500},
"TypeError":{"message":"Internal Server Error","error_code":500},
"ValueError":{"message":"Internal Server Error","error_code":500},
"Warning":{"message":"Warning","error_code":200},
"KeyboardInterrupt":{"message":"Keyboard Interrupt","error_code":500},
"SystemExit":{"message":"Internal Server Error","error_code":500},
}

def exception_handler(msg):
    print("msg args",msg.args)
    import traceback
    tb = traceback.format_exc()
    print("tb------>>", tb)
    logger.error("%s",msg.__class__.__name__)
    logger.error("%s", msg.args[0])
    logger.error("%s", tb)
    if msg.__class__.__name__ == "KeyError" and msg.args[0] == "HTTP_UID":
            return jsonify({'error': 'Authentication Error, Uid is missing'}), 400
    else:
        error = exception_hash.get(msg.__class__.__name__ ,{"message":"Internal Server Error","error_code":500})
        return jsonify(error["message"]), error["error_code"]



##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
import time
import json
from pymongo import MongoClient
import datetime
from dateutil.parser import parse
import os, sys, inspect, importlib
from api import settings,users
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir)
from backend.ysocket import *
from backend.tcpclient import *
from backend import header
from backend import rabbitmq_config

# ysocket.connectionstatus = 1

connection = ""
channel = ""
mongoClient = ""
user = ""
active_user = ""

def dbdump(mongoClient,active_user, body):
    # body = json.loads(body)
    print(body)
    if "tabname" in body:
        status = False
        for i in range(len(rabbitmq_config.queues)):
            if body["tabname"] == rabbitmq_config.queues[i]["name"]+"_"+active_user and "efficiency" not in body["tabname"]:
                db = mongoClient.xregv
                try:
                    db[rabbitmq_config.queues[i]["config"]["dbname"]].insert_one(body)
                except Exception as e:
                    print("errror--->>", str(e))
                status = True
                break
            else:
                status = False
        if not status:
            if "efficiency" in body["tabname"]:
                db = mongoClient.xregv
                try:
                    db.ddr_efficiency.insert_one(body)
                except Exception as e:
                    print("errror--->>", str(e))
            else:
                db = mongoClient.xregv
                try:
                    db.socketdata.insert_one(body)
                except Exception as e:
                    print("errror--->>", str(e))




##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
import tornado.httpserver
import tornado.websocket
import tornado.ioloop
import tornado.web
import socket
import json,datetime
from bson import json_util,ObjectId
import pymongo
import os, sys, inspect, importlib
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir)
from api import settings
from time import sleep                                                                                 
from api import users
from backend import rabbitmq_config

def myconverter(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()

class WSHandler(tornado.websocket.WebSocketHandler):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.id = "wsocket"
        self.mongo = pymongo.MongoClient(settings.MONGO_URI)
        self.db = self.mongo.xregv

    def open(self):
        print('new connection')
        print('Host: %s' % self.request.host)
        print('Entire uri: %s' % self.request.uri)
        print('Uri path: %s' % self.request.path)
        print('Query path w/o ?: %s' % self.request.query)
        print(self.request.remote_ip)
        query_args = dict(self.request.query_arguments)     
        query_args = {k: v[0].decode("utf-8") for k,v in query_args.items()}
        if query_args.get("socketpath") == "browserhandler":
            uniquekey = query_args.get("uniquekey")
            users.set_user(uniquekey, {"clientconnectionstatus": True})

        self.id = self.request.uri.split("/")
        self.id = self.id[1]

    def on_message(self, message):
        try:
            body= json.loads(message)
            time = datetime.datetime.now()
            time = time.timestamp()
            if ("tabname" in body and body["tabname"].lower() in ["plot","overlay"]) or ("tab" in body and body["tab"].lower() in ["plot","overlay"]):
                self.write_message(json.dumps(self.customisegraphdata(body,time), default=myconverter))
            elif ("tabname" in body and body["tabname"].lower() in ["syswatch"]) or ("tab" in body and body["tab"].lower() in ["syswatch"]):
                self.write_message(json.dumps(self.watchgraph(body,time), default=myconverter))
            elif ( "tab" in body and body["tab"].lower() in ["active_connection"]):                                                                   
                self.write_message(json.dumps(self.checkconnection(body), default=myconverter))
            else:
                self.write_message(json.dumps(self.performancegraph(body,time), default=myconverter))
        except Exception as e:
            print("error", str(e))
            import traceback
            import traceback
            tb = traceback.format_exc()
            print("tb------>>", tb)

    def checkconnection(self, body):
        if "tab" in body:
            newdata = [{"tab": body["tab"], "data": [" message received"]}]
            return newdata
        else:
            return {"error":"Mandatory key missing"}

    def customisegraphdata(self, body, time):
        if "threads" in body:
            finaldata = []
            for i in body["threads"]:
                match = {"tabname":body["tab"]+"_"+body["uid"],"threadname":i["threadname"],"time":{"$gt":float(time)}}
                count = 1
                if "time" in i and i["time"]!= "":
                    time = i['time']
                    match["time"] = {"$gt":float(time)}
                data = list(self.db.overlaydb.find(match).sort([("time", pymongo.ASCENDING)]).limit(count))
                newdata = {"threadname":i["threadname"],"tabname": body["tab"]+"_"+body["uid"], "data": [], "time": time}
                if len(data):
                    newdata["tabname"] = data[0]["tabname"]
                    newdata["time"] = str(data[len(data) - 1]["time"])
                    for x in range(len(data)):
                        if "error" in data[x]:
                            newdata["error"] = data[x]["error"]
                        else:
                            for m in range(len(data[x]["data"])):
                                newdata["data"].append(data[x]["data"][m])
                finaldata.append(newdata)
            return finaldata
        else:
            return {"error":"Mandatory key missing"}

    def watchgraph(self, body, time):
        if "tabname" in body and "threadname" in body:
            match = {"tabname":body["tabname"],"threadname":body["threadname"],"time":{"$gt":float(time)}}
            count = 1
            if "time" in body and body["time"]!= "":
                time = body['time']
                match["time"] = {"$gt":float(time)}
            data = list(self.db.socketdata.find(match).sort([("time", pymongo.ASCENDING)]).limit(count))
            newdata = [{"tabname": body["tabname"], "data": [], "time": time, "threadname": body["threadname"]}]
            if len(data):
                newdata[0]["time"] = str(data[len(data) - 1]["time"])
                for x in range(len(data)):
                    for n in range(len(newdata)):
                        if newdata[n]["threadname"] == data[x]["threadname"]:
                            for m in range(len(data[x]["data"])):
                                newdata[n]["data"].append(data[x]["data"][m])
                            break
                        else:
                            if n == len(newdata)-1:
                                datum = []
                                for m in range(len(data[x]["data"])):
                                    datum.append(data[x]["data"][m])
                                newdata.append({"tabname": data[x]["tabname"], "data": datum, "time": str(data[len(data) - 1]["time"]), "threadname": data[x]["threadname"]})
                    if "error" in data[x]:
                        newdata["error"] = data[x]["error"]
            return newdata
        else:
            return {"error":"Mandatory key missing"}

    def performancegraph(self, body, time):
        if "tab" in body:
            matchnotfound = True
            for i in range(len(rabbitmq_config.queues)):
                if body["tab"] == rabbitmq_config.queues[i]["name"] or ("ddr_efficiency" in body["tab"] and rabbitmq_config.queues[i]["name"] == "ddr_efficiency"):
                    matchnotfound = False
                    match = {"tabname": body["tab"] + "_" + body["uid"], "time": {"$gte": float(time)}}
                    tabname_emptydata = body["tab"] + "_" + body["uid"]
                    if "threadname" in body:
                        threadname_emptydata = body["threadname"]
                    else:
                        threadname_emptydata = rabbitmq_config.queues[i]["config"]["tabname"] + "_" + body["uid"]
                    count = 1
                    if "time" in body and body["time"]!= "":
                        time = body['time']
                        match["time"] = {"$gt":float(time)}
                        count = 100
                    if "threadname" in body:
                        match["threadname"] = {"$in":body["threadname"]}
                    if "thread_createtime" in body:
                        match["thread_createtime"] = {"$in":body["thread_createtime"]}
                    data = list(self.db[rabbitmq_config.queues[i]["config"]["dbname"]].find(match).sort([("time", pymongo.ASCENDING)]).limit(count))
                    newdata = [{"threadname":threadname_emptydata,"tabname": tabname_emptydata, "data": [], "time": time}]
                    if len(data):
                        newdata[0]["time"] = str(data[len(data) - 1]["time"])
                        for x in range(len(data)):
                            if "error" in data[x]:
                                newdata[0]["error"] = data[x]["error"]
                            else:
                                for m in range(len(data[x]["data"])):
                                    newdata[0]["data"].append(data[x]["data"][m])
                    return newdata
                else:
                    matchnotfound = True
            if matchnotfound:
               return {"error":"No performance found"} 
        else:
            return {"error":"Mandatory key missing"}


    def send_message(self, message):
        self.write_message(message)

    def on_close(self):
        print('connection closed')
        query_args = dict(self.request.query_arguments)
        query_args = {k: v[0].decode("utf-8") for k,v in query_args.items()}
        if query_args.get("socketpath") == "browserhandler":
            uniquekey = query_args.get("uniquekey")
            users.set_user(uniquekey, {"clientconnectionstatus": False,"checkmodule_request": False})
        # self.application.remove_event_listener(self)

    def check_origin(self, origin):
        return True


application = tornado.web.Application([
    (r'/ws', WSHandler)
])

io_loop = tornado.ioloop.IOLoop.instance()
http_server = tornado.httpserver.HTTPServer(application)
http_server.listen(49999)
io_loop.start()



##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
#!/usr/bin/env python
import os,sys,inspect,importlib,json,datetime
import threading
from multiprocessing import Process

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
apidir = os.path.join(parentdir,"api")
backdir = os.path.join(parentdir,"backend")
sys.path.insert(0,parentdir)
from api import users,settings
from backend import ysocket
from backend import tcpclient
# from backend import ysocket
from backend import header
from backend import rabbitmq_config
from backend import receiver
from time import sleep
import string
import logging
# import numpy as np
from pymongo import MongoClient

# reduce log level
connection=""
channel = ""


def myconverter(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()

class PikaClient:
    def __init__(self, threadname="",client="",active_user="",tab="",readperformance="",delay=0.5,tabstatus=True,log="",baseaddress="",steps="",location="",tempobj = ""):
        self.threadname = threadname
        self.performancethread = ""
        self.tab = tab
        self.readperformance = readperformance
        self.delay = delay
        self.client = client
        self.active_user = active_user
        self.tabstatus = tabstatus
        self.log = log
        self.connectionstatus = 0
        self.baseaddress = baseaddress
        self.steps = steps
        self.location = location
        self.tempobj = tempobj
        self.sample = 0
        self.thread_createtime = datetime.datetime.now().timestamp()

    def updateconnectionstatus(self):
        user = users.get_user(self.active_user, "connectionstatus")
        self.connectionstatus = user["connectionstatus"]
        
    def createthread(self):
        self.performancethread = threading.Thread(target=self.newconnection, args=())
        self.performancethread.deamon = True
        self.performancethread.start()

    def newconnection(self):
        self.updateconnectionstatus()
        mongoClient = MongoClient(settings.MONGO_URI)
        if self.tab == "overlay" and self.connectionstatus and self.tabstatus:
            createoverlayfile(self.readperformance,self.log,self.active_user,self.client,self.tempobj,self.threadname)
        while self.connectionstatus and self.tabstatus:
            self.updateconnectionstatus()
            stat = True
            if self.tab == "overlay":
                message =readoverlay(self.log,self.active_user,self.client,self.threadname,self.thread_createtime)
            else:
                for i in range(len(rabbitmq_config.queues)):
                    if rabbitmq_config.queues[i]["config"]["tabname"].lower() == self.tab.lower():
                        if "efficiency" in rabbitmq_config.queues[i]["config"]["tabname"].lower():
                            message = self.getefficiency(rabbitmq_config.queues[i],self.log,self.active_user,self.client,self.threadname)
                        else:
                            message = eval(self.readperformance)(rabbitmq_config.queues[i],self.log,self.active_user,self.client,self.threadname)
                        stat = False
                        break
                if stat:
                    message = self.readsysaiwireless()
            receiver.dbdump(mongoClient,self.active_user,message)
            sleep(float(self.delay))
            print("delay", self.delay)
            print("time", datetime.datetime.now())
        print("closing channel")
        mongoClient.close()
        return 1

    def readsysaiwireless(self):
        sysaiwirelessArr = {"threadname":self.threadname,"tabname": self.tab + "_" + self.active_user, "data": {}}
        sysaiwirelessArr["data"]["key"] = self.threadname
        sysaiwirelessArr["data"]["time"] = datetime.datetime.now().timestamp()
        sysaiwirelessArr["data"]["data"] = []
        sysaiwirelessArr["time"] = sysaiwirelessArr["data"]["time"]
        if self.baseaddress != "" and "0x" in self.baseaddress:
            dataarr = []
            skipindex = int(int(self.steps) / 4)
            if int(self.location) > 0:
                address = self.baseaddress
                try:
                    tempdata = (tcpclient.runcommandfromui(("mrd %s %s" % (address, str(self.location))), self.log,self.client))
                except Exception as e:
                    print(str(e))
                dataarr = dataarr + tempdata
            tempdataarr = dataarr[0::skipindex]
            diff_logic = 0
            for i in range(len(tempdataarr)):
                temp = tempdataarr[i].split(':')
                if len(temp) == 2:
                    if all(c in 'xX' + string.hexdigits for c in temp[1].strip()):
                        if("0x8010" in self.baseaddress):
                            diff_logic = 1
                            a = temp[1].strip()
                            imgI = "0x%s" % a[0:4]
                            realI = "0x%s" % a[4:8]
                            i1 = np.int16(np.int(imgI, 16))
                            r1 = np.int16(np.int(realI, 16))
                            sysaiwirelessArr["data"]["data"].append({"address":temp[0].strip(),"y":int(np.sqrt(pow(i1, 2) + pow(r1, 2) ))})
                        else:
                            temp[1] = temp[1].strip()
                            temp[1] = "0x" + temp[1][4:]
                            new_val = np.int16(np.int(temp[1], 16))
                            temp[1] = int(new_val)
                            # sysaiwirelessArr["data"]["data"].append({"address":temp[0].strip(),"value":temp[1]})
                            sysaiwirelessArr["data"]["data"].append({"address":temp[0].strip(),"y":temp[1]})
                # address = hex(int(address,16)+int(self.steps))
        if diff_logic == 1:
            maxabs = max(sysaiwirelessArr["data"]["data"])
            newdata = [round(x/maxabs, 1) for x in sysaiwirelessArr["data"]["data"]]
            sysaiwirelessArr["data"]["data"] = newdata
        return sysaiwirelessArr
    def getefficiency(self, exchange, log, active_user, client,threadname):
        try:
            perfArr = {"threadname":threadname, "tabname": "ddr_efficiency_"+exchange["name"].split("_efficiency")[0] + "_" + active_user, "data": []}
            for i in range(len(exchange["config"]["instances"])):
                filename = os.path.join(apidir, "%s.tcl" % (str(exchange["config"]["file_name"])))
                mrdNoc = open(filename, "r")
                lines = mrdNoc.readlines()
                mrdNoc.close()
                timenow = datetime.datetime.now().timestamp()
                if self.sample != 0:
                    self.sample = timenow - self.sample
                else:
                    self.sample = timenow
                perfArr["time"] = timenow
                for line in lines:
                    keys = line.replace("\n", "").split(",")
                    newobj = {}
                    command_data = tcpclient.runcommandfromui("mrd %s 3" % keys[5], log, client)
                    tcpclient.runcommandfromui("mwr %s 0x0 3" % keys[5], log, client)
                    if (type(command_data) is list) and (len(command_data) >= 3):
                        newobj = {'accumulatedlatency': command_data[0].replace(' ', '').split(':')[-1],
                                  'burstcount': command_data[1].replace(' ', '').split(':')[-1],
                                  'transactioncount': command_data[2].replace(' ', '').split(':')[-1]
                                  }
                        newobj["throughput"] = int(newobj["transactioncount"], 16) * int(newobj["burstcount"], 16) * 1024 * (
                                    self.sample / 1000)
                        newobj["theoritical_throughput"] = 24
                        newobj["efficiency"] = newobj["throughput"] / newobj["theoritical_throughput"]
                        newobj["time"] = timenow
                        perfArr["data"].append({'module': keys[0],
                                                'submodule': keys[1],
                                                'monitor': keys[2],
                                                'key': "_".join(keys[0:3]),
                                                'data': [newobj]
                                                })
                    else:
                        perfArr["error"] = "Error on reading data from board"
            return perfArr
        except Exception as e:
            import traceback
            tb = traceback.format_exc()
            print("tb------>>", tb)

def set_bit(v, index, x):
  """Set the index:th bit of v to 1 if x is truthy, else to 0, and return the new value."""
  v = int(v, 16)
  mask = 1 << index   # Compute mask, an integer with just bit 'index' set.
  v &= ~mask          # Clear the bit indicated by the mask (if x is False)
  if x:
    v |= mask         # If x was True, set the bit indicated by the mask.
  p = hex(v)[2:]
  while len(p) <8:
      p = str(0)+p
  return p

def getperf(exchange,log,active_user,client,threadname):
    perfArr = {"threadname":threadname,"tabname":exchange["name"]+"_"+active_user, "data":[]}
    for i in range(len(exchange["config"]["instances"])):
        filename = os.path.join(apidir, "%s.tcl" %(str(exchange["config"]["file_name"])))
        mrdNoc = open(filename, "r")
        lines = mrdNoc.readlines()
        mrdNoc.close()
        timenow = datetime.datetime.now().timestamp()
        perfArr["time"] = timenow
        for line in lines:
            keys = line.replace("\n", "").split(",")
            newobj = {}
            command_data = tcpclient.runcommandfromui("mrd %s 3" % keys[5], log, client)
            if (type(command_data) is list) and (len(command_data) >= 3):
                newobj = {'accumulatedlatency': command_data[0].replace(' ', '').split(':')[-1],
                          'burstcount': command_data[1].replace(' ', '').split(':')[-1],
                          'transactioncount': command_data[2].replace(' ', '').split(':')[-1]
                         }
                overflow = []
                for key, val in newobj.items():
                    if ( int(val, 16) & (1<<31) ):
                        overflow.append(key)
                    newobj[key] = str("%x" % (int(val,  16) & int("0x7FFFFFFF", 16)))

                newobj.update({'overflow': overflow,
                               'time': timenow
                              })
                perfArr["data"].append({'module': keys[0],
                                        'submodule': keys[1],
                                        'monitor': keys[2],
                                        'key': "_".join(keys[0:3]),
                                        'data': [newobj]
                                       })
            else:
                perfArr["error"] = "Error on reading data from board"
    return perfArr

def getlpdcounters(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir,str(header.READ_LPD_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"lpd_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(
                        ("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                        {"time": timenow, str(keys[0]): str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr
def getpmccounters(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir,str(header.READ_PMC_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"pmc_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(
                        ("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] ="Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                    {"time": timenow, str(keys[0]): str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getfpdcounters(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir,str(header.READ_FPD_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"fpd_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[0]): str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getcpmcounters(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir,str(header.READ_CPM_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"cpm_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] ="Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                    {"time": timenow, str(keys[0]): str("%x" % (int(str(tempvar),  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getcpmnsu(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir, str(header.READ_CPM_NSU))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname": "cpm_nsu_" + active_user, "data": []}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log, client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str(
                            "%x" % (int(str(tempvar), 16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log, client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                        {"time": timenow, str(keys[0]): str("%x" % (int(str(tempvar), 16) & int("0x0000FFFF", 16))),
                         "overflow": []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getdcperformance(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir, str(header.PERF_DDR_DC))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"dc_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    overflow = []
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        if (int(str(tempvar),  16) & (1<<31)):
                            counterArr["data"][i]["data"][0]["overflow"].append(str(keys[0]))
                        else:
                            val = str(keys[0])
                            counterArr["data"][i]["data"][0]["overflow"] = [value for value in (counterArr["data"][i]["data"][0]["overflow"]) if value != val]
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(str(tempvar),  16) & int("0x7FFFFFFF", 16)))
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    if (int(str(tempvar),  16) & (1<<31)):
                        overflow.append(str(keys[0]))
                    else:
                        val = str(keys[0])
                        overflow = [value for value in overflow if value != val]
                    obj["data"].append({"time":timenow, str(keys[0]): str("%x" % (int(str(tempvar),  16) & int("0x7FFFFFFF", 16))), "overflow" : overflow})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getapmperformance(exchange,log,active_user,client,threadname):
    filename = os.path.join(apidir,str(header.READ_APM))
    readcounters = open(filename, "r")
    counterArr = {"threadname":threadname,"tabname":"pl_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time": timenow, str(keys[0]): tempvar})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getnocperformance(exchange,log,active_user,client,threadname):
    counterArr = {"threadname":threadname,"tabname":"noc_"+active_user,"data":[]}
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    dataobj = {}
    filename = os.path.join(apidir, str(header.READ_NMU_REG))
    readnmu = open(filename, "r")
    for line in readnmu:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if (len(keys) >= 3) and (line[0] != "#"):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]) and counterArr["data"][i]["key"] == "nmu_"+str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False
            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = "nmu_"+str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[0]):tempvar, "Accumulated_latency":""})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
            for i in range(len(counterArr["data"])):
                if (counterArr["data"][i]["data"][0]) and ("Accumulated_latency_upper" in counterArr["data"][i]["data"][0]) and ("Accumulated_latency_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["Accumulated_latency"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["Accumulated_latency_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["Accumulated_latency_lower"])
                if (counterArr["data"][i]["data"][0]) and ("byte_count_upper" in counterArr["data"][i]["data"][0]) and ("byte_count_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["byte_count"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["byte_count_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["byte_count_lower"])
    readnmu.close()
    status = False
    filename = os.path.join(apidir,str(header.READ_NSU_REG))
    readnsu = open(filename, "r")
    for line in readnsu:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if (len(keys) >= 3) and (line[0] != "#"):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]) and counterArr["data"][i]["key"] == "nsu_"+str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = "nsu_"+str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[0]):tempvar})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
            for i in range(len(counterArr["data"])):
                if (counterArr["data"][i]["data"][0]) and ("Accumulated_latency_upper" in counterArr["data"][i]["data"][0]) and ("Accumulated_latency_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["Accumulated_latency"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["Accumulated_latency_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["Accumulated_latency_lower"])
                if (counterArr["data"][i]["data"][0]) and ("byte_count_upper" in counterArr["data"][i]["data"][0]) and ("byte_count_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["byte_count"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["byte_count_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["byte_count_lower"])
    readnsu.close()

    npsdataarr = []
    for j in range(0, ysocket.numberofnps):
        status = False
        filename = os.path.join(apidir, str("%s"%header.READ_NPS_REG+str(j)+".tcl"))
        readnps = open(filename, "r")
        for line in readnps:
            line = line.replace("\n", "")
            keys = line.split(" ")
            if (len(keys) >= 3) and (line[0] != "#"):
                for i in range(len(counterArr["data"])):
                    if  counterArr["data"][i]["key"] == "nps_"+str(j):
                        status = True
                        if (str(keys[0]) == "counter_1"):
                            prevvalue = counterArr["data"][i]["data"][0][str(keys[2])]
                            tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])),log,client)
                            if all(c in 'xX' + string.hexdigits for c in tempvar):
                                counterArr["data"][i]["data"][0][str(keys[2])] = str(set_bit(tempvar, 31, 0))
                                counterArr["data"][i]["data"][0][str(keys[2])] = counterArr["data"][i]["data"][0][
                                                                                     str(keys[2])] + "" + prevvalue
                            else:
                                counterArr["error"] = "Error on reading data from board"
                        else:
                            tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                            if all(c in 'xX' + string.hexdigits for c in tempvar):
                                counterArr["data"][i]["data"][0][str(keys[2])] = tempvar
                            else:
                                counterArr["error"] = "Error on reading data from board"
                        break
                    else:
                        status = False

                if not status:
                    obj = {}
                    obj["data"] = []
                    obj["displayname"] = str(keys[0])
                    obj["key"] = "nps_"+str(j)
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[1])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        obj["data"].append({"time":timenow, str(keys[2]):tempvar})
                        counterArr["data"].append(obj)
                    else:
                        counterArr["error"] = "Error on reading data from board"
        readnps.close()
    return counterArr

def getmeperf(exchange,log,active_user,client,threadname):
    counterArr = {"threadname":threadname,"tabname":"aiengine_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    status = False

    for j in range(0,len(client.mebaseaddress)):
        timenow = datetime.datetime.now().timestamp()
        counterArr["time"] = timenow
        filename = os.path.join(apidir, str(header.READ_ME))
        readcounters = open(filename, "r")
        for line in readcounters:
            line = line.replace("\n", "")
            keys = line.split(" ")
            if ((len(keys) >= 3) and (line[0] != '#')):
                for i in range(len(counterArr["data"])):
                    if counterArr["data"][i]["displayname"] == str(keys[2])+str(j):
                        status = True
                        address = hex(int(client.mebaseaddress[j], 16) + int(str(keys[1]), 16))
                        tempvar = tcpclient.getcommandresponse(("mrd %s\n" %address),log,client)
                        if all(c in 'xX' + string.hexdigits for c in tempvar):
                            counterArr["data"][i]["data"][0][str(keys[0])] = tempvar
                        else:
                            counterArr["error"] = "Error on reading data from board"
                        break
                    else:
                        status = False

                if not status:
                    obj = {}
                    obj["data"] = []
                    obj["displayname"] = str(keys[2]) + str(j)
                    obj["key"] = str(keys[2])+str(j)
                    address = hex(int(client.mebaseaddress[j], 16) + int(str(keys[1]), 16))
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" %address),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        obj["data"].append({"time": timenow, str(keys[0]): tempvar})
                        counterArr["data"].append(obj)
                    else:
                        counterArr["error"] = "Error on reading data from board"
        readcounters.close()
    return counterArr

#create .tcl file for read overlay performance
def createoverlayfile(readperformance,log,active_user,client,tempobj,threadname):
    methodarr = readperformance.split("/")
    open(threadname+".tcl", 'w').close()
    newfp = open(threadname+".tcl","a")
    for i in range(len(methodarr)):
        for j in range(len(rabbitmq_config.queues)):
            if methodarr[i] != "" and methodarr[i] == rabbitmq_config.queues[j]["config"]["method"]:
                filename = os.path.join(apidir, str(getattr(header,rabbitmq_config.queues[j]["config"]["file_name"])))
                fp = open(filename, "r")
                for performanceindex in range(len(tempobj["performance"])):
                    if tempobj["performance"][performanceindex]["subtab"] == rabbitmq_config.queues[j]["name"]:
                        for line in fp:
                            line = line.replace("\n", "")
                            if methodarr[i]!= "getperf":
                                keys = line.split(" ")
                                if ((len(keys) >= 3) and (line[0] != '#')):
                                    if any(x in line for x in tempobj["performance"][performanceindex]["plot"]):
                                        line = keys[0]+"_"+keys[2]+" "+keys[1]+" "+tempobj["key"]
                                        newfp.write(line + "\n")
                            else:
                                keys = line.split(",")
                                if len(keys)==6:
                                    mrdkeys = ["accumulatedlatency","burstcount","transactioncount"]
                                    for x in range(0,3):
                                        keys[3] = mrdkeys[x]
                                        line = keys[0]+"_"+keys[1]+"_"+keys[2]+" "+keys[5]+" "+tempobj["key"]
                                        if ((len(keys) >= 3) and (line[0] != '#')):
                                            if any(x in line for x in tempobj["performance"][performanceindex]["plot"]):
                                                newfp.write(line + "\n")

                fp.close()
    newfp.close()

def readoverlay(log,active_user,client,threadname,thread_createtime):
    filename = os.path.join(apidir, str(threadname+".tcl"))
    readcounters = open(filename, "r")
    counterArr = {"tabname": "overlay_" + active_user, "data": [],"threadname":threadname,"thread_createtime":thread_createtime}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now().timestamp()
    counterArr["time"] = timenow
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log, client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log, client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time": timenow, str(keys[0]): tempvar})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr


##
# Copyright (c) 2021, Xilinx Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
##
# MONGO_URI = "mongodb://vncmgr:vncw0rld19@xsj-pvdbvnc02:27060,xsj-pvdbvnc03:27060,xsj-pvdbvnc04:27060/xregv?authSource=admin&readPreference=secondaryPreferred"
MONGO_URI = "mongodb://localhost:27017/xregv"
SECRET_KEY = 'the quick brown fox jumps over the lazy dog'
CORS_HEADERS = 'Content-Type'
ENVIRONMENT = 'production'

URI = 'ldap://ldap.xilinx.com:389'
LDAP_DOMAIN = 'xilinx.com'
LDAP_SEARCH_BASE = 'OU=people,DC=xilinx,DC=com'

# DB_Lookup = {"S80_ES1_VnC":{"PS":"xregv_ps_S80_ES1_VnC","PMC":"xregv_ps_S80_ES1_VnC","CPM":"xregv_ps_S80_ES1_VnC","NPI":"xregv_npi_S80_ES1_VnC","PL":"xregvpl_S80_ES1_VnC","AI_ENGINE":"xregvme_S80_ES1_VnC","DDR":"xregv_npi_S80_ES1_VnC"},
# "S80_ES1_VCK190":{"PS":"xregv_ps_S80_ES1_VCK180","PMC":"xregv_ps_S80_ES1_VCK180","CPM":"xregv_ps_S80_ES1_VCK180","NPI":"xregv_npi_S80_ES1_VCK180","PL":"xregvpl_S80_ES1_VnC","AI_ENGINE":"xregvme_S80_ES1_VnC","DDR":"xregv_npi_S80_ES1_VCK180"},
# "S80_ES2_VnC":{"PS":"xregv_ps_S80_ES2_VnC","PMC":"xregv_ps_S80_ES2_VnC","CPM":"xregv_ps_S80_ES2_VnC","NPI":"xregv_npi_S80_ES1_VnC","PL":"xregvpl_S80_ES1_VnC","AI_ENGINE":"xregvme_S80_ES1_VnC","DDR":"xregv_npi_S80_ES1_VnC"}}

DB_Lookup = {"S80_ES1_VCK190":{"PS":"xregv_ps_S80_ES1_VCK180","AM012":"xregv_ps_S80_ES1_VCK180","PMC":"xregv_ps_S80_ES1_VCK180","CPM":"xregv_ps_S80_ES1_VCK180","NPI":"xregv_npi_S80_ES1_VCK180","PL":"xregvpl_S80_ES1_VnC","AI_ENGINE":"xregvme_S80_ES1_VnC","DDR":"xregv_npi_S80_ES1_VCK180"}}



##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
#!/usr/bin/env python
import os,sys,inspect,importlib,json,datetime
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
apidir = os.path.join(parentdir,"api")
sys.path.insert(0,parentdir)
from backend import ysocket
from backend import tcpclient
# from backend import ysocket
from backend import header
from backend import rabbitmq_config
from time import sleep
import string
import logging
import threading
from multiprocessing import Process
threadarr = []

# reduce log level

connection=""
channel = ""
def myconverter(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()

def set_bit(v, index, x):
  """Set the index:th bit of v to 1 if x is truthy, else to 0, and return the new value."""
  v = int(v, 16)
  mask = 1 << index   # Compute mask, an integer with just bit 'index' set.
  v &= ~mask          # Clear the bit indicated by the mask (if x is False)
  if x:
    v |= mask         # If x was True, set the bit indicated by the mask.
  p = hex(v)[2:]
  while len(p) <8:
      p = str(0)+p
  return p

def getperf(exchange,log,active_user,client):
    perfArr = {"tabname":exchange["name"]+"_"+active_user, "data":[]}
    for i in range(len(exchange["config"]["instances"])):
        filename = os.path.join(apidir,str(exchange["config"]["file_name"] + str(exchange["config"]["instances"][i]) + ".tcl"))
        mrdNoc = open(filename, "r")
        timenow = datetime.datetime.now()
        overflow = []
        for line in mrdNoc:
            line = line.replace("\n", "")
            keys = line.split(",")
            perfmonData = {}
            perfmonData["data"] = []
            newobj = {}
            newobj["accumulatedlatency"] = (
                tcpclient.getcommandresponse(("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("0", 16)))), log,client))
            newobj["burstcount"] = (
                tcpclient.getcommandresponse(("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("4", 16)))), log,client))
            newobj["transactioncount"] = (
                tcpclient.getcommandresponse(("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("8", 16)))), log,client))
            newobj["time"] = timenow
            perfmonData["module"] = keys[0]
            perfmonData["submodule"] = keys[1]
            perfmonData["monitor"] = keys[2]
            perfmonData["key"] = perfmonData["module"]+"_"+perfmonData["submodule"]+"_"+perfmonData["monitor"]
            if all(c in 'xX' + string.hexdigits for c in newobj["accumulatedlatency"]) and all(c in 'xX' + string.hexdigits for c in newobj["burstcount"]) and all(c in 'xX' + string.hexdigits for c in newobj["transactioncount"]):
                if (int(newobj["accumulatedlatency"],  16) & (1<<31)):
                    overflow.append("accumulatedlatency")
                else:
                    val = "accumulatedlatency"
                    overflow = [value for value in overflow if value != val]
                if (int(newobj["burstcount"],  16) & (1<<31)):
                    overflow.append("burstcount")
                else:
                    val = "burstcount"
                    overflow = [value for value in overflow if value != val]
                if (int(newobj["transactioncount"],  16) & (1<<31)):
                    overflow.append("transactioncount")
                else:
                    val = "transactioncount"
                    overflow = [value for value in overflow if value != val]
                newobj["accumulatedlatency"] = str("%x" % (int(newobj["accumulatedlatency"],  16) & int("0x7FFFFFFF", 16)))
                newobj["burstcount"] = str("%x" % (int(newobj["burstcount"], 16) & int("0x7FFFFFFF", 16)))
                newobj["transactioncount"] = str("%x" % (int(newobj["transactioncount"], 16) & int("0x7FFFFFFF", 16)))
                newobj["overflow"] = overflow
                perfmonData["data"].append(newobj)
                perfArr["data"].append(perfmonData)
            else:
                perfArr["error"] = "Error on reading data from board"

    mrdNoc.close()
    return perfArr


def getlpdcounters(exchange,log,active_user,client):
    filename = os.path.join(apidir,str(header.READ_LPD_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"lpd_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(
                        ("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                        {"time": timenow, str(keys[0]): str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr
def getpmccounters(exchange,log,active_user,client):
    filename = os.path.join(apidir,str(header.READ_PMC_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"pmc_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(
                        ("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] ="Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                    {"time": timenow, str(keys[0]): str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getfpdcounters(exchange,log,active_user,client):
    filename = os.path.join(apidir,str(header.READ_FPD_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"fpd_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[0]): str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getcpmcounters(exchange,log,active_user,client):
    filename = os.path.join(apidir,str(header.READ_CPM_COUNTERS))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"cpm_"+active_user,"data":[]}
    counterData = {}
    newobj = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[2]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[0])] = str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16)))
                        counterArr["data"][i]["data"][0]["overflow"] = []
                    else:
                        counterArr["error"] ="Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[2])
                obj["key"] = str(keys[2])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append(
                    {"time": timenow, str(keys[0]): str("%x" % (int(tempvar,  16) & int("0x0000FFFF", 16))), "overflow" : []})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getdcperformance(exchange,log,active_user,client):
    filename = os.path.join(apidir, str(header.PERF_DDR_DC))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"dc_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    overflow = []
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[0]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        if (int(tempvar,  16) & (1<<31)):
                            counterArr["data"][i]["data"][0]["overflow"].append(str(keys[2]))
                        else:
                            val = str(keys[2])
                            counterArr["data"][i]["data"][0]["overflow"] = [value for value in (counterArr["data"][i]["data"][0]["overflow"]) if value != val]
                        counterArr["data"][i]["data"][0][str(keys[2])] = str("%x" % (int(tempvar,  16) & int("0x7FFFFFFF", 16)))
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[0])
                obj["key"] = str(keys[0])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[1])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    if (int(tempvar,  16) & (1<<31)):
                        overflow.append(str(keys[2]))
                    else:
                        val = str(keys[2])
                        overflow = [value for value in overflow if value != val]
                    obj["data"].append({"time":timenow, str(keys[2]): str("%x" % (int(tempvar,  16) & int("0x7FFFFFFF", 16))), "overflow" : overflow})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getapmperformance(exchange,log,active_user,client):
    filename = os.path.join(apidir,str(header.READ_APM))
    readcounters = open(filename, "r")
    counterArr = {"tabname":"pl_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    timenow = datetime.datetime.now()
    status = False
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if ((len(keys) >= 3) and (line[0] != '#')):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[1]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[0])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[2])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[1])
                obj["key"] = str(keys[1])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[0])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time": timenow, str(keys[2]): tempvar})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
    readcounters.close()
    return counterArr

def getnocperformance(exchange,log,active_user,client):
    counterArr = {"tabname":"noc_"+active_user,"data":[]}
    timenow = datetime.datetime.now()
    status = False
    dataobj = {}
    filename = os.path.join(apidir, str(header.READ_NMU_REG))
    readnmu = open(filename, "r")
    for line in readnmu:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if (len(keys) >= 3) and (line[0] != "#"):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[1]) and counterArr["data"][i]["key"] == "nmu_"+str(keys[1]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[2])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False
            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[1])
                obj["key"] = "nmu_"+str(keys[1])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[2]):tempvar, "Accumulated_latency":""})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
            for i in range(len(counterArr["data"])):
                if (counterArr["data"][i]["data"][0]) and ("Accumulated_latency_upper" in counterArr["data"][i]["data"][0]) and ("Accumulated_latency_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["Accumulated_latency"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["Accumulated_latency_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["Accumulated_latency_lower"])
                if (counterArr["data"][i]["data"][0]) and ("byte_count_upper" in counterArr["data"][i]["data"][0]) and ("byte_count_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["byte_count"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["byte_count_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["byte_count_lower"])
    readnmu.close()
    status = False
    filename = os.path.join(apidir,str(header.READ_NSU_REG))
    readnsu = open(filename, "r")
    for line in readnsu:
        line = line.replace("\n", "")
        keys = line.split(" ")
        if (len(keys) >= 3) and (line[0] != "#"):
            for i in range(len(counterArr["data"])):
                if counterArr["data"][i]["displayname"] == str(keys[1]) and counterArr["data"][i]["key"] == "nsu_"+str(keys[1]):
                    status = True
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])), log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        counterArr["data"][i]["data"][0][str(keys[2])] = tempvar
                    else:
                        counterArr["error"] = "Error on reading data from board"
                    break
                else:
                    status = False

            if not status:
                obj = {}
                obj["data"] = []
                obj["displayname"] = str(keys[1])
                obj["key"] = "nsu_"+str(keys[1])
                tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])), log,client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    obj["data"].append({"time":timenow, str(keys[2]):tempvar})
                    counterArr["data"].append(obj)
                else:
                    counterArr["error"] = "Error on reading data from board"
            for i in range(len(counterArr["data"])):
                if (counterArr["data"][i]["data"][0]) and ("Accumulated_latency_upper" in counterArr["data"][i]["data"][0]) and ("Accumulated_latency_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["Accumulated_latency"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["Accumulated_latency_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["Accumulated_latency_lower"])
                if (counterArr["data"][i]["data"][0]) and ("byte_count_upper" in counterArr["data"][i]["data"][0]) and ("byte_count_lower" in counterArr["data"][i]["data"][0]):
                    counterArr["data"][i]["data"][0]["byte_count"] = str(str("%x" % (int(str(counterArr["data"][i]["data"][0]["byte_count_upper"]), 16) & int("0x000000FF", 16))) +
                     counterArr["data"][i]["data"][0]["byte_count_lower"])
    readnsu.close()

    npsdataarr = []
    for j in range(0, ysocket.numberofnps):
        status = False
        filename = os.path.join(apidir, str("%s"%header.READ_NPS_REG+str(j)+".tcl"))
        readnps = open(filename, "r")
        for line in readnps:
            line = line.replace("\n", "")
            keys = line.split(" ")
            if (len(keys) >= 3) and (line[0] != "#"):
                for i in range(len(counterArr["data"])):
                    if  counterArr["data"][i]["key"] == "nps_"+str(j):
                        status = True
                        if (str(keys[2]) == "counter_1"):
                            prevvalue = counterArr["data"][i]["data"][0][str(keys[1])]
                            tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])),log,client)
                            if all(c in 'xX' + string.hexdigits for c in tempvar):
                                counterArr["data"][i]["data"][0][str(keys[1])] = str(set_bit(tempvar, 31, 0))
                                counterArr["data"][i]["data"][0][str(keys[1])] = counterArr["data"][i]["data"][0][
                                                                                     str(keys[1])] + "" + prevvalue
                            else:
                                counterArr["error"] = "Error on reading data from board"
                        else:
                            tempvar = tcpclient.getcommandresponse(("mrd %s\n" % str(keys[0])),log,client)
                            if all(c in 'xX' + string.hexdigits for c in tempvar):
                                counterArr["data"][i]["data"][0][str(keys[1])] = tempvar
                            else:
                                counterArr["error"] = "Error on reading data from board"
                        break
                    else:
                        status = False

                if not status:
                    obj = {}
                    obj["data"] = []
                    obj["displayname"] = str(keys[2])
                    obj["key"] = "nps_"+str(j)
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n"% str(keys[0])),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        obj["data"].append({"time":timenow, str(keys[1]):tempvar})
                        counterArr["data"].append(obj)
                    else:
                        counterArr["error"] = "Error on reading data from board"
        readnps.close()
    return counterArr

def getmeperf(exchange,log,active_user,client):
    counterArr = {"tabname":"aiengine_"+active_user, "data":[]}
    counterData = {}
    counterData["data"] = []
    status = False

    for j in range(0,len(client.mebaseaddress)):
        timenow = datetime.datetime.now()
        filename = os.path.join(apidir, str(header.READ_ME))
        readcounters = open(filename, "r")
        for line in readcounters:
            line = line.replace("\n", "")
            keys = line.split(" ")
            if ((len(keys) >= 3) and (line[0] != '#')):
                for i in range(len(counterArr["data"])):
                    if counterArr["data"][i]["displayname"] == str(keys[1])+str(j):
                        status = True
                        address = hex(int(client.mebaseaddress[j], 16) + int(str(keys[0]), 16))
                        tempvar = tcpclient.getcommandresponse(("mrd %s\n" %address),log,client)
                        if all(c in 'xX' + string.hexdigits for c in tempvar):
                            counterArr["data"][i]["data"][0][str(keys[2])] = tempvar
                        else:
                            counterArr["error"] = "Error on reading data from board"
                        break
                    else:
                        status = False

                if not status:
                    obj = {}
                    obj["data"] = []
                    obj["displayname"] = str(keys[1]) + str(j)
                    obj["key"] = str(keys[1])+str(j)
                    address = hex(int(client.mebaseaddress[j], 16) + int(str(keys[0]), 16))
                    tempvar = tcpclient.getcommandresponse(("mrd %s\n" %address),log,client)
                    if all(c in 'xX' + string.hexdigits for c in tempvar):
                        obj["data"].append({"time": timenow, str(keys[2]): tempvar})
                        counterArr["data"].append(obj)
                    else:
                        counterArr["error"] = "Error on reading data from board"
        readcounters.close()
    return counterArr

def dumpperformancedata(active_user='',client="",constatus = 0):
    tcpclient.configuremonitors(client)
    tcpclient.configuredcmonitors(client)
    tcpclient.configurelpdCounters(client)
    tcpclient.configurefpdCounters(client)
    tcpclient.configureapmmonitors(client)
    tcpclient.configurecpmcounters(client)
    tcpclient.configurepmccounters(client)

    while client.connectionstatus:
        for i in range(len(rabbitmq_config.queues)):
            if rabbitmq_config.queues[i]['config']['method'] == 'getlpdcounters':
                message = getlpdcounters(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getfpdcounters':
                message = getfpdcounters(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getcpmcounters':
                message = getcpmcounters(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getpmccounters':
                message = getpmccounters(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getperf':
                message = getperf(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getdcperformance':
                message = getdcperformance(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getapmperformance':
                message = getapmperformance(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            elif rabbitmq_config.queues[i]['config']['method'] == 'getmeperf':
                message = getmeperf(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)
            if ysocket.numberofnps and rabbitmq_config.queues[i]['config']['method'] == 'getnocperformance':
                message = getnocperformance(rabbitmq_config.queues[i],rabbitmq_config.queues[i]['config']['logfile'],active_user,client)

            body = message
            print(body)
            if "tabname" in body:
                for i in range(len(rabbitmq_config.queues)):
                    if body["tabname"] == rabbitmq_config.queues[i]["name"]+"_"+active_user and "efficiency" not in body["tabname"]:
                        db = mongoClient.xregv
                        try:
                            db[rabbitmq_config.queues[i]["config"]["dbname"]].insert_one(body)
                        except Exception as e:
                            print("errror--->>", str(e))
                if "efficiency" in body["tabname"]:
                    db = mongoClient.xregv
                    try:
                        db.ddr_efficiency.insert_one(body)
                    except Exception as e:
                        print("errror--->>", str(e))

if __name__ == '__main__':
    dumpperformancedata()


##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
#!/usr/bin/env python
import os,sys,inspect,importlib,json,datetime
import threading
from multiprocessing import Process

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
sys.path.insert(0,parentdir)
from api import users,settings
from backend import ysocket
from backend import tcpclient
# from backend import ysocket
from backend import header
from backend import rabbitmq_config
from backend import receiver
from pymongo import MongoClient
from time import sleep
import string
import logging

# reduce log level
connection=""
channel = ""

def myconverter(o):
    if isinstance(o, datetime.datetime):
        return o.__str__()

class PikaClient:
    def __init__(self, threadname="",hw_name="",module_name="",register_name="",bit_field="",address="",legend="",displayname="",y_axis="",client="",active_client=""):
        self.threadname = threadname
        self.hw_name = hw_name
        self.module_name = module_name
        self.register_name = register_name
        self.bit_field = bit_field
        self.address = address
        self.legend = legend
        self.y_axis = y_axis
        self.plotthread = ""
        self.displayname = displayname
        self.client = client
        self.plotstatus = True
        self.active_client = active_client
        self.thread_createtime = datetime.datetime.now().timestamp()

    def createthread(self):
        self.plotthread = threading.Thread(target=self.newconnection, args=())
        self.plotthread.deamon = True
        self.plotthread.start()

    def newconnection(self):
        # global connection, channel
        mongoClient = MongoClient(settings.MONGO_URI)
        try:
            user = users.get_user(self.active_client, "connectionstatus")
            while user["connectionstatus"] and self.plotstatus:
                user = users.get_user(self.active_client, "connectionstatus")
                message = self.getlpdcounters(self.threadname)
                receiver.dbdump(mongoClient,self.active_client, message)           
            print("closing channel")
        finally:
            mongoClient.close()
        return 1

    def getlpdcounters(self,exchange):
        counterArr = {"tabname": "plot_"+self.active_client,"threadname":self.threadname,"y_axis":self.y_axis, "data": [],"thread_createtime":self.thread_createtime}
        counterData = {}
        newobj = {}
        counterData["data"] = []
        timenow = datetime.datetime.now().timestamp()
        counterArr["time"] = timenow

        status = False
        for i in range(len(counterArr["data"])):
            if counterArr["data"][i]["displayname"] == self.displayname:
                status = True
                tempvar = tcpclient.getcommandresponse(
                    ("mrd %s\n" % self.address), "plot",self.client)
                if all(c in 'xX' + string.hexdigits for c in tempvar):
                    counterArr["data"][i]["data"][0][self.legend] = str(
                        "%x" % (int(tempvar, 16)))
                else:
                    counterArr["error"] = "Error on reading data from board"
                break
            else:
                status = False

        if not status:
            obj = {}
            obj["data"] = []
            obj["displayname"] = self.displayname
            obj["key"] = self.threadname
            tempvar = tcpclient.getcommandresponse(("mrd %s\n" % self.address), "plot",self.client)
            if all(c in 'xX' + string.hexdigits for c in tempvar):
                obj["data"].append({"time": timenow, self.legend: str("%x" % (int(tempvar, 16)))})
                counterArr["data"].append(obj)
            else:
                counterArr["error"] = "Error on reading data from board"
        return counterArr


##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
#
##
#!/usr/bin/env python

import os, sys, inspect, importlib,string,re
from backend import ysocket
from backend import header
import json
import time
import threading
from api import settings
from pymongo import MongoClient
from multiprocessing import Process
import datetime
from backend import *
# from backend import sender
# from backend import receiver
from backend import rabbitmq_config
from backend import performanceview

currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
apidir = os.path.join(parentdir, "api")
backdir = os.path.join(parentdir, "backend")
client = ysocket.SocketClient('', '', '')
performancethread = []  # threading.Thread(target = getperformance, args=())
dbdump = []
configarr = []

def isHex(val):
    if (val.startswith("0x") or val.startswith("0X")) and set(val[2:]).issubset(string.hexdigits):
        return True
    else:
        return False

def connecttoserver(hostname, port, active_user):
    global performancethread, dbdump
    client = ysocket.SocketClient(hostname, port, active_user)
    if (client):
        if (not client.connect()):
            ysocket.boardname = hostname
            ysocket.xsdbport = port
            # dbdumpthread = threading.Thread(target=receiver.recvData, args=(active_user,))
            # dbdumpthread.deamon = True
            # dbdumpthread.start()
            return client
        else:
            return 0


def disconnectserver(client):
    global performancethread, dbdump
    # sender.closeconnection()
    client.connectionstatus = 0
    ysocket.numberofnps = 0
    time.sleep(1)
    client.disconnect()
    print("socket disconnected")
    ysocket.boardname = ""
    ysocket.xsdbport = ""
    return 1

def configaws(client):
    configFile = open(os.path.join(apidir, "%s" % header.ConfigAWS), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "ConfigAWS")
            print("resp",resp)
    configFile.close()

def getvoltage(dataarr,resp,key_name):
    print(dataarr,"\n")
    print("resp",resp)
    exponent_mask = int("0x60000",16)
    format_mask = int("0x10000",16)
    mantissa_mask = int("0xFFFF",16)
    for i in range(len(dataarr)):
        temp = dataarr[i].split(":")
        if len(temp) == 2:
            my_addr_h = "0x"+temp[0].strip()
            my_number = int(temp[1].strip(),16)
            exponent = ((my_number & exponent_mask) >> 17) - 16
            format_m = (my_number & format_mask)   >> 16
            mantissa = my_number & mantissa_mask
            if format_m & ((mantissa) >> 15):
                my_decimal = (mantissa * pow(2,exponent)) - 1
            else:
                my_decimal = mantissa * pow(2,exponent)
            my_decimal_5f = float("{0:.5f}".format(my_decimal))
            status = True
            if len(resp)<14:
                for j in range(len(resp)):
                    if resp[j]["name"] == header.AWS_lookup[my_addr_h]:
                        resp[j][key_name] = my_decimal_5f
                        status = False
                        break
                    else:
                        status = True
                if status:
                    obj = {}
                    obj["name"] = header.AWS_lookup[my_addr_h]
                    obj[key_name] = my_decimal_5f
                    resp.append(obj)
            else:
                resp[i][key_name] = my_decimal_5f
                if key_name not in ["min", "max"]:
                    resp[i][key_name+"_per"] = my_decimal_5f*100
    # print("resp",resp)
    return resp
def readaws(client):
    resp = []
    temp = runcommandfromui("mrd 0xF1271040 10","ReadAWS",client)
    resp = getvoltage(temp,resp,"value")
    temp = runcommandfromui("mrd 0xF1271340 10","ReadAWS",client)
    resp = getvoltage(temp,resp,"min")
    temp = runcommandfromui("mrd 0xF1271640 10","ReadAWS",client)
    resp = getvoltage(temp,resp,"max")
    # print(resp)
    for i in range(len(resp)):
        resp[i]["value_per"] = str(resp[i]["value"]/(resp[i]["min"]+resp[i]["max"])*100) + "%"
        resp[i]["threshold"] = 0
        resp[i]["threshold_per"] = str(0)+"%"
    return resp
    
def readaws_old(client):
    resp = []
    temp = runcommandfromui("mrd 0xF1271040 14","ReadAWS",client)
    resp = getvoltage(temp,resp,"value")
    temp = runcommandfromui("mrd 0xF1271340 14","ReadAWS",client)
    resp = getvoltage(temp,resp,"min")
    temp = runcommandfromui("mrd 0xF1271640 14","ReadAWS",client)
    resp = getvoltage(temp,resp,"max")
    # print(resp)
    for i in range(len(resp)):
        resp[i]["value_per"] = str(resp[i]["value"]/(resp[i]["min"]+resp[i]["max"])*100) + "%"
        resp[i]["threshold"] = 0
        resp[i]["threshold_per"] = str(0)+"%"
    return resp


def configureapmmonitors(client):
    # global client
    configFile = open(os.path.join(apidir, "%s" % header.CONF_APM), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "APM_PERF")
    configFile.close()
    print("APM perfmon config done")


def configuredcmonitors(client):
    # global client
    print("dc perfmon config start")
    configFile = open(os.path.join(apidir, "%s" % header.CONF_DC_PERF), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "DC_PERF")
    configFile.close()
    print("dc perfmon config done")

def configuremonitors0(client):
    print("counter config 0 start")
    configFile = open(os.path.join(apidir, "%s" % header.CONF_DDR_NOC_0), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "DDR_NOC_0_PERF")
    configFile.close()
    print("counter config 0 done")

def configuremonitors1(client):
    print("counter config 1 start")
    configFile = open(os.path.join(apidir, "%s" % header.CONF_DDR_NOC_1), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "DDR_NOC_1_PERF")
    configFile.close()
    print("counter config 1 done")

def configuremonitors2(client):
    print("counter config 2 start")
    configFile = open(os.path.join(apidir, "%s" % header.CONF_DDR_NOC_2), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "DDR_NOC_2_PERF")
    configFile.close()
    print("counter config 2 done")

def configuremonitors3(client):
    print("counter config 3 start")
    configFile = open(os.path.join(apidir, "%s" % header.CONF_DDR_NOC_3), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "DDR_NOC_3_PERF")
    configFile.close()
    print("counter config 3 done")

def resetLpdCounters(client):
    configFile = open(os.path.join(apidir, "%s" % header.RESET_LPD_COUNTERS), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()

def configurelpdCounters(client):
    # global client
    print("lpd counter config start")
    resetLpdCounters(client)
    configFile = open(os.path.join(apidir, "%s" % header.PS_PMC_RPU_AXI0), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTLPD_AFIFSLPD_AXI), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTLPD_OCM_AXI), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTLPD_INTFPD_AXI), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.XPPU), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "LPD_PERF")
    configFile.close()
    print("lpd counter config done")
    return 0

def resetFpdCounters(client):
    configFile = open(os.path.join(apidir, "%s" % header.RESET_FPD_COUNTERS), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()

def configurefpdCounters(client):
    # global client
    print("fpd counter config start")
    resetFpdCounters(client)
    configFile = open(os.path.join(apidir, "%s" % header.FPD_SLAVE), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTFLD_INTLPD_OCM), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTFPD_INTLPD_LPD_SLAVE), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.INTFPD_INTLPD_PCIE), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.PS_FPD_APM), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.PS_NOC_NCI_AXI0), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.PS_NOC_NCI_AXI1), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "FPD_PERF")
    configFile.close()
    print("fpd counter config done")


def configurecpmcounters(client):
    # global client
    print("cpm counter config start")
    configFile = open(os.path.join(apidir, "%s" % header.CPM_AXI0), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "CPM_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.CPM_AXI1), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "CPM_PERF")
    configFile.close()
    configFile = open(os.path.join(apidir, "%s" % header.CPM_PCIe), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "CPM_PERF")
    configFile.close()
    print("cpm counter config done")

def resetPmcCounters(client):
    configFile = open(os.path.join(apidir, "%s" % header.RESET_PMC_COUNTERS), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "PMC_PERF")
    configFile.close()

def configurepmccounters(client):
    # global client
    print("pmc counter config start")
    resetPmcCounters(client)
    configFile = open(os.path.join(apidir, "%s" % header.PMC_NOC_AXI), "r")
    for line in configFile:
        if ((line[0] != "#") and (len(line) > 2)):
            resp = client.send_msg(line, "PMC_PERF")
    configFile.close()
    print("pmc counter config done")


def getperformance(client="", active_user="", conf=""):
    for j in range(len(conf)):
        conf_found_status = False
        for i in range(len(rabbitmq_config.queues)):
            if rabbitmq_config.queues[i]["config"]["tabname"].lower() == conf[j]["displayname"].lower():
                conf_found_status = True
                delay = conf[j].get("delay",0.1)
                if conf[j]["permission"]:
                    activethreadstatus = False
                    threadname = rabbitmq_config.queues[i]["config"]["tabname"] + "_" + active_user
                    for threads in performancethread:
                        if threadname == threads["threadname"]:
                            activethreadstatus = True
                            conf[j]["threadname"] = threadname
                            # threads["dependency"].append("performance")
                            break
                        else:
                            activethreadstatus = False
                    if not activethreadstatus:
                        # eval(rabbitmq_config.queues[i]["config"]["configuration"])(client)
                        if "configuration" in rabbitmq_config.queues[i]["config"]:
                            print("configuration found", rabbitmq_config.queues[i]["config"]["configuration"])
                            eval(rabbitmq_config.queues[i]["config"]["configuration"])(client)

                            configarr.append({"configuration":rabbitmq_config.queues[i]["config"]["configuration"],"board":client.host, "port":client.port,"uid":active_user,"threadname":rabbitmq_config.queues[i]["config"]["tabname"] + "_" + active_user})

                        try:
                            newthread = performanceview.PikaClient(threadname, client, active_user,
                                                               rabbitmq_config.queues[i]["config"]["tabname"],
                                                               rabbitmq_config.queues[i]["config"]["method"],
                                                               delay, conf[j]["status"],
                                                               rabbitmq_config.queues[i]["config"]["logfile"])
                            # performancethread.append({"threadname": newthread.threadname, "thread": newthread,"dependency":["performance"]})
                            performancethread.append({"threadname": newthread.threadname, "thread": newthread})
                            newthread.createthread()
                            conf[j]["threadname"] = newthread.threadname
                        except Exception as e:
                            print("error---------->>>", str(e))
                            conf[j]["error"] = "Thread not created"
        if not conf_found_status:
            conf[j]["error"] = "Performance not found"
    # dbdumpthread = threading.Thread(target=receiver.recvData, args=(active_user,))
    # dbdumpthread.deamon = True
    # dbdumpthread.start()
    return conf

def getoverlay(client="", active_user="", conf=""):
    print("client ", client, "active_user", active_user, "conf", conf)
    for length in range(len(conf)):
        overlay = {"name": "overlay",
                   "config": {
                       "method": "",
                       "dbname": "overlaydb",
                       "tabname": "overlay",
                       "logfile": "overlay",
                   }
                   }
        #[{"tab":"overlay","performance":[{"subtab":"PMC","plot":["PMC_0","PMC_1"]},{"subtab":"DDRMC_0","plot":["NSU_1","NSU_2"]}],"uid":"pranayd","threadname":"overlay_1_pranayd"}]
        if "performance" in conf[length] and len(conf[length]["performance"])>0:
            for k in range(len(conf[length]["performance"])):
                for i in range(len(rabbitmq_config.queues)):
                    if rabbitmq_config.queues[i]["name"].lower() == conf[length]["performance"][k]["subtab"].lower():
                        # subthreadname = rabbitmq_config.queues[i]["config"]["tabname"] + "_" + active_user
                        activethreadstatus= False
                        for configure in configarr:
                            if (configure["configuration"] == rabbitmq_config.queues[i]["config"]["configuration"]) and (configure["board"] == client.host) and (configure["port"] == client.port) and (configure.uid == active_user):
                                print("thread found----", threads)
                                activethreadstatus = True
                                # threads["dependency"].append("overlay")
                                break
                            else:
                                activethreadstatus = False
                        if not activethreadstatus:
                            eval(rabbitmq_config.queues[i]["config"]["configuration"])(client) #writing configuration
                            configarr.append({"configuration":rabbitmq_config.queues[i]["config"]["configuration"],"board":client.host, "port":client.port,"uid":active_user})
                        overlay["config"]["method"] += rabbitmq_config.queues[i]["config"]["method"]+"/" #all read method
        threadname = conf[length]["threadname"]
        try:
            newthread = performanceview.PikaClient(threadname, client, active_user,
                                                   overlay["config"]["tabname"],
                                                   overlay["config"]["method"],
                                                   0.1, True,
                                                   overlay["config"]["logfile"],tempobj=conf[length])
            performancethread.append({"threadname": newthread.threadname, "thread": newthread})
            newthread.createthread()
            conf[length]["threadname"] = newthread.threadname
            conf[length]["thread_createtime"] = newthread.thread_createtime
        except Exception as e:
            print("error---------->>>", str(e))
            conf[length]["error"] = "Thread not created"
        print("conf------>>>",conf)
    return conf


def getperf(num, client):
    mrdNoc = open(os.path.join(apidir, str("perf_ddr_noc_" + str(num) + ".tcl")), "r")
    perfArr = []
    timenow = datetime.datetime.now()
    for line in mrdNoc:
        line = line.replace("\n", "")
        keys = line.split(",")
        perfmonData = {}
        perfmonData["accumulatedlatency"] = getcommandresponse(
            ("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("0", 16)))), "perf_ddr_noc", client)
        perfmonData["burstcount"] = getcommandresponse(
            ("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("4", 16)))), "perf_ddr_noc", client)
        perfmonData["transactioncount"] = getcommandresponse(
            ("mrd 0x%s\n" % str("%x" % (int(keys[5], 16) + int("8", 16)))), "perf_ddr_noc", client)
        perfmonData["module"] = keys[0]
        perfmonData["submodule"] = keys[1]
        perfmonData["monitor"] = keys[2]
        perfmonData["time"] = timenow
        perfArr.append(perfmonData)
    mongoClient = MongoClient(settings.MONGO_URI)
    db = mongoClient.xregv
    db.perfmon_1.insert_one(perfArr)
    mrdNoc.close()


def getlpdcounters(client):
    readcounters = open(os.path.join(apidir, "%s" % header.READ_LPD_COUNTERS), "r")
    counterArr = []
    timenow = datetime.datetime.now()
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        counterData = {}
        counterData["time"] = timenow
        if ((len(keys) >= 3) and (line[0] != '#')):
            counterData["requesttype"] = str(keys[0])
            counterData["value"] = getcommandresponse(("mrd %s\n" % str(keys[1])), "perf_LPD", client)
            counterData["displayname"] = str(keys[2])
            counterArr.append(counterData)
    mongoClient = MongoClient(settings.MONGO_URI)
    db = mongoClient.xregv
    db.lpdcounters.insert_one(counterArr)
    readcounters.close()


def getfpdcounters(client):
    readcounters = open(os.path.join(apidir, "%s" % header.READ_FPD_COUNTERS), "r")
    counterArr = []
    timenow = datetime.datetime.now()
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        counterData = {}
        counterData["time"] = timenow
        if ((len(keys) >= 3) and (line[0] != '#')):
            counterData["requesttype"] = str(keys[0])
            counterData["value"] = getcommandresponse(("mrd %s\n" % str(keys[1])), "perf_FPD", client)
            counterData["displayname"] = str(keys[2])
            counterArr.append(counterData)
    mongoClient = MongoClient(settings.MONGO_URI)
    db = mongoClient.xregv
    db.fpdcounters.insert_one(counterArr)
    readcounters.close()


def getcpmcounters(client):
    readcounters = open(os.path.join(apidir, "%s" % header.READ_CPM_COUNTERS), "r")
    counterArr = []
    timenow = datetime.datetime.now()
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        counterData = {}
        counterData["time"] = timenow
        if ((len(keys) >= 3) and (line[0] != '#')):
            counterData["requesttype"] = str(keys[0])
            counterData["value"] = getcommandresponse(("mrd %s\n" % str(keys[1])), "perf_CPM", client)
            counterData["displayname"] = str(keys[2])
            counterArr.append(counterData)
    mongoClient = MongoClient(settings.MONGO_URI)
    db = mongoClient.xregv
    db.cpmcounters.insert_one(counterArr)
    readcounters.close()


def getpmccounters(client):
    readcounters = open(os.path.join(apidir, "%s" % header.READ_PMC_COUNTERS), "r")
    counterArr = []
    timenow = datetime.datetime.now()
    for line in readcounters:
        line = line.replace("\n", "")
        keys = line.split(" ")
        counterData = {}
        counterData["time"] = timenow
        if ((len(keys) >= 3) and (line[0] != '#')):
            counterData["requesttype"] = str(keys[0])
            counterData["value"] = getcommandresponse(("mrd %s\n" % str(keys[1])), "perf_PMC", client)
            counterData["displayname"] = str(keys[2])
            counterArr.append(counterData)
    mongoClient = MongoClient(settings.MONGO_URI)
    db = mongoClient.xregv
    db.pmccounters.insert_one(counterArr)
    readcounters.close()


def getcommandresponse(command, cmdsrc, client, target="",beam_target=""):
    resp = str(client.send_msg(command, cmdsrc, target,beam_target))
    if ((resp.find("error") == -1) and ("error" != resp) and (len(resp) > 1)):
        # data = resp.replace("okay ", "").split(':')
        data = re.sub('^["error","okay"]*', '', resp)
        data = data.split(':')
        if len(data) >= 2:
            return " ".join(((data[1]).replace("\\r", "").replace("\\n", "").replace("\\", "").replace("\'", "")).split())
    resp = re.sub('^["error","okay"]*', '', resp)
    return " ".join((resp.replace("\\r", "").replace("\\n", "").replace("\\", "").replace("\'", "")).split())


def runcommandfromui(command, cmdsrc, client, target="",beam_target=""):
    # global client
    resp = str(client.send_msg(command + "\n", cmdsrc, target,beam_target))
    if (("error" != resp) and (len(resp) > 1)):
        data = re.sub('^["error","okay"]*', '', resp)
        data = (" ".join((data.replace("\\r", "").replace("\'", "")).split())).split("\\n")
        for x in range(0, len(data)):
            data[x] = re.sub('^["error","okay"]*', '', data[x])
            data[x] = " ".join((data[x].replace("\\", "").replace("\"", "")).split())
        return data
    else:
        return 0


def getregistervalue(address, client, target="",beam_target=""):
    return getcommandresponse(("mrd %s\n" % address), "getRegister", client, target,beam_target)


def rundicommands(command, cmdsrc, client, target="",beam_target="", is_tcl_script=False):
    # global client
    resp = str(client.di_send_msg(command + "\n", cmdsrc, is_tcl_script, target,beam_target))
    if (("error" != resp) and (len(resp) > 1)):
        data = re.sub('^["error","okay"]*', '', resp)
        data = (" ".join((data.replace("\\r", "").replace("\'", "")).split())).split("\\n")
        for x in range(0, len(data)):
            data[x] = re.sub('^["error","okay"]*', '', data[x])
            data[x] = " ".join((data[x].replace("\\", "").replace("\"", "")).split())
        return data
    else:
        return 0



##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
#
##
import os,inspect
import re
from pymongo import MongoClient
currentdir = os.path.dirname(os.path.abspath(inspect.getfile(inspect.currentframe())))
parentdir = os.path.dirname(currentdir)
apidir = os.path.join(parentdir,"api")

client = MongoClient("mongodb://localhost:27017/xregv")
db=client["xregv"]

version="xregv_ps_S80_ES1_VCK180"

AM012 = {
    "hw_name": "AM012",
    "instance_name": [
        "CFRAME00_REG",
        "APU_GIC_DIST_MAIN",
        "CFRAME02_REG",
        "APU_GIC_REDIST_CTLLPI_0",
        "APU_GIC_REDIST_SGISPI_0",
        "CFRAME01_REG",
        "APU_GIC_ITS_TRANS",
        "APU_GIC_CPUIF",
        "APU_GIC_REDIST_SGISPI_1",
        "APU_GIC_DIST_MBSPI",
        "APU_GIC_VIFCTL",
        "APU_DUAL_CSR",
        "CANFD0",
        "CANFD1",
        "APU_GIC_VCPUIF",
        "APU_GIC_ITS_CTL",
        "APU_GIC_REDIST_CTLLPI_1",
        "DPC_DMA_CSR",
        "CFRAME04_REG",
        "CFRAME08_REG",
        "FPD_CCI_CSR",
        "FPD_SLCR_SECURE",
        "FPD_CCI_CORE",
        "LPD_DMA_CH5",
        "FPD_INT_CSR",
        "LPD_DMA_CH0",
        "FPD_SMMU_CSR",
        "CFRAME05_REG",
        "CFRAME03_REG",
        "CFRAME11_REG",
        "CFRAME07_REG",
        "LPD_DMA_CH1",
        "LPD_DMA_CH3",
        "CFRAME09_REG",
        "LPD_DMA_CH2",
        "CFRAME13_REG",
        "RPU_DUAL_CSR",
        "CRL",
        "CRP",
        "LPD_DMA_CH4",
        "RPU_GIC_PL390",
        "CFRAME_BCAST_REG",
        "FPD_SLCR",
        "CFRAME10_REG",
        "CFRAME12_REG",
        "CFRAME14_REG",
        "CFU_CSR",
        "DPC_AURORA",
        "CFRAME06_REG",
        "CRF",
        "TTC0",
        "LPD_IOP_SLCR_SECURE",
        "PPU_ICACHE_CTRL",
        "PMC_DMA0_CSR",
        "LPD_GPIO",
        "PMC_JTAG_CSR",
        "LPD_IOP_SLCR",
        "LPD_SLCR_SECURE",
        "SD_eMMC1",
        "XRAM_XMPU2",
        "DBG_APU0_ETM",
        "DBG_FPD_CTI1B",
        "PMC_INT_GPV",
        "DBG_CPM_ELA2A",
        "DBG_LPD_GPR",
        "DBG_FPD_REPL",
        "LPD_IOP_INT_GPV",
        "PL_AXI_LPD_CSR",
        "PPU_DCACHE_CTRL",
        "PMC_GLOBAL",
        "PMC_IOP_SLCR",
        "PSM_INT_GPV",
        "XRAM_CTRL1",
        "PSM_GLOBAL",
        "LPD_INT_CSR",
        "PPU_TMR_INJECT",
        "PLM_RTCA",
        "PMC_GPIO",
        "PMC_SBI_CSR",
        "PMC_XPPU",
        "UART1",
        "XRAM_XMPU3",
        "DBG_CPM_ELA2B",
        "XRAM_CTRL3",
        "DBG_CPM_ROM",
        "DBG_PMC_GPR",
        "DBG_RPU0_DBG",
        "IPI",
        "SPI0",
        "SD_eMMC0",
        "SPI1",
        "TTC2",
        "PL_AXI_FPD_CSR",
        "DBG_APU1_CTI",
        "PPU_MDM",
        "PMC_IOP_SLCR_SECURE",
        "PMC_RTC",
        "XRAM_INT_GPV",
        "OSPI",
        "PSM_TMR_TRACE",
        "DBG_APU1_ETM",
        "LPD_I2C1",
        "XRAM_SLCR",
        "LPD_INT_GPV",
        "DBG_RPU1_CTI",
        "DBG_CPM_ELA2C",
        "SCNTR",
        "LPD_SLCR",
        "PSM_TMR_INJECT",
        "PMC_EFUSE_CTRL",
        "PMC_IOP_INT_GPV",
        "PSM_LOCAL",
        "XRAM_XMPU0",
        "SCNTR_SECURE",
        "FPD_SWDT",
        "PL_ACELITE_FPD_CSR",
        "PSM_MDM",
        "PSM_ICACHE_ECC",
        "PSM_TMR_MANAGER",
        "DBG_CPM_ELA2D",
        "PMC_LOCAL",
        "PMC_RAM_CSR",
        "USB2_XHCI",
        "LPD_DMA_CH6",
        "DBG_APU1_DBG",
        "DBG_FPD_FUN",
        "DBG_FPD_GPR",
        "OCM_CSR",
        "LPD_XPPU",
        "QSPI",
        "LPD_I2C0",
        "PMC_I2C",
        "UART0",
        "PMC_XMPU",
        "XRAM_CTRL2",
        "DBG_PMC_CTI",
        "DBG_FPD_CTI1C",
        "PPU_TMR_MANAGER",
        "XRAM_XMPU1",
        "XRAM_CTRL0",
        "DBG_APU1_PMU",
        "DBG_CPM_CTI2D",
        "DBG_RPU1_ETM",
        "DBG_RPU0_ETM",
        "LPD_DMA_CH7",
        "GEM1",
        "GEM0",
        "TTC1",
        "TTC3",
        "PPU_TMR_TRACE",
        "PMC_ANLG",
        "PMC_INT_CSR",
        "PMC_SYSMON_CSR",
        "DBG_APU0_DBG",
        "DBG_APU0_PMU",
        "PPU_IOMODULE",
        "PSM_IOMODULE",
        "PSM_DCACHE_ECC",
        "PMC_EFUSE_CACHE",
        "PSM_INT_CSR",
        "USB2_CSR",
        "DBG_LPD_CTI",
        "DBG_APU_CTI",
        "DBG_FPD_ROM",
        "DBG_LPD_FUN",
        "DBG_PMC_ROM",
        "DBG_APU0_CTI",
        "DBG_RPU0_CTI",
        "DBG_FPD_CTI1D",
        "DBG_APU_ELA",
        "DBG_FPD_ETR",
        "DBG_RPU1_DBG",
        "LPD_SWDT",
        "PMC_DMA1_CSR",
        "PMC_XPPU_NPI",
        "FPD_XMPU",
        "OCM_XMPU",
        "DBG_CPM_CTI2A",
        "DBG_FPD_ETF",
        "DBG_APU_ETF",
        "DBG_LPD_ROM",
        "CPM4_ADDRREMAP",
        "CPM4_XDMA_CSR",
        "CPM4_DVSEC1",
        "CPM5_INT_CSR",
        "DBG_TSG_RW",
        "CPM5_CRX",
        "CPM4_DVSEC0",
        "CPM5_INT_GPV",
        "CPM4_L2_CSR",
        "CPM4_DMA_ATTR",
        "DBG_CPM_FUN",
        "CPM4_CRX",
        "CPM4_INT_CSR",
        "CPM5_SLCR",
        "CPM5_SLCR_SECURE",
        "CPM_PCSR",
        "CPM5_CMN",
        "CPM4_PCIE0_ATTR",
        "CPM4_SLCR",
        "CPM5_L20_CSR",
        "CPM5_ADDRREMAP",
        "DBG_FPD_TPIU",
        "CPM4_INT_GPV",
        "DBG_STM",
        "DBG_APU_FUN",
        "CPM4_SLCR_SECURE",
        "CPM5_PCIE0_CSR",
        "CPM4_CMN",
        "CPM4_PCIE1_ATTR",
        "CPM5_PCIE0_ATTR",
        "CPM5_L21_CSR",
        "CPM5_PCIE1_CSR",
        "CPM5_DVSEC0",
        "CPM5_DMA0_ATTR",
        "CPM5_PCIE1_ATTR",
        "CPM5_DPLL0_ATTR",
        "CPM5_DMA1_ATTR",
        "CPM5_DVSEC1",
        "CPM5_DPLL1_ATTR",
        "CPM5_DMA1_CSR",
        "CPM5_DMA0_CSR"
    ]
}

DB_Lookup = db["db_lookup"]
DB_Lookup = DB_Lookup.find_one()
blocks = [AM012]
for dataDict in blocks:
    block_name= dataDict["hw_name"]
    db1="xregv_ps_S80_ES1_VCK180"

    common_path = apidir
    outputname = ""
    block = f"{dataDict['hw_name']}"
    block_path = os.path.join(common_path, block)
    if not os.path.exists(block_path):
        os.makedirs(block_path)
    if "hw_name" in dataDict:
        outputname = outputname + dataDict["hw_name"]
    if dataDict["hw_name"] == "DDR":
        matchfilter = {"instance_name": {"$in": [re.compile(r'DDRMC'), re.IGNORECASE]}}
    if dataDict["hw_name"] == "NPI":
        matchfilter = {"instance_name": {"$nin": [re.compile(r'DDR'), re.IGNORECASE]}}
    if dataDict["hw_name"] == "PMC":
        matchfilter = {"instance_name": {"$in": [re.compile(r'PMC'), re.IGNORECASE]}}
    if dataDict["hw_name"] == "CPM":
        matchfilter = {"instance_name": {
            "$in": [re.compile(r'CPM'), re.compile(r'XDMA'), re.compile(r'PCIEA'), re.IGNORECASE, ]}}
    if dataDict["hw_name"] == "PS":
        matchfilter = {"instance_name": {
            "$nin": [re.compile(r'PMC'), re.compile(r'CPM'), re.compile(r'XDMA'), re.compile(r'PCIEA'),
                     re.IGNORECASE, ]}}
    if "instance_name" in dataDict and type(dataDict["instance_name"]) != list:
        outputname = outputname + "_" + dataDict["instance_name"]
        matchfilter = {"instance_name": dataDict["instance_name"]}
    if "instance_name" in dataDict and type(dataDict["instance_name"]) == list:
        matchfilter = {"instance_name": {"$in": dataDict["instance_name"]}}

    db1 = db[db1]
    resultcursor = list(db1.aggregate([{"$match": matchfilter}, {
        "$project": {"_id": 0, "instance_name": 1, "base_address": 1, "registers": 1}},
                                      {"$unwind": "$registers"}, {
                                          "$project": {"registers.name": 1, "registers.address": 1,
                                                       "instance_name": 1, "base_address": 1,
                                                       "registers.fields": 1,
                                                       "registers.reset_value": 1, "registers.width": 1}}]))
    instance_data = {}
    for entry in resultcursor:
        instance_name = entry['instance_name']
        if instance_name not in instance_data:
            instance_data[instance_name] = []
        instance_data[instance_name].append(entry)
    for instance_name, data_list in instance_data.items():
        tcl_content = ""
        base_address = data_list[0]['base_address']
        for entry in data_list:
            register_name = entry['registers']['name']
            address = entry['registers']['address']
            reset_value = entry['registers']['reset_value']
            absolute_address = hex(int(base_address, 16) + int(address, 16))
            if int(entry['registers']['width']) < 32 and absolute_address[-1] not in ['0','4','8','c']:
                tcl_content += f"lappend results [mem_read16 {absolute_address}]\n"
            else:
                tcl_content += f"lappend results [mem_read {absolute_address}]\n"

        os.chdir(block_path)
        with open(f"{instance_name}.tcl", "w") as tcl_file:
            tcl_file.write(f"# Instance: {instance_name}\n")
            tcl_file.write("set results {}\n")
            tcl_file.write("proc mem_read {add} {\n")
            tcl_file.write("    if {[catch {mrd $add} err]} {\n")
            tcl_file.write("        return \"$add: $err\"\n")
            tcl_file.write("    } else {\n")
            tcl_file.write("        return $err\n")
            tcl_file.write("    }\n")
            tcl_file.write("}\n")
            tcl_file.write("proc mem_read16 {add} {\n")                                                   
            tcl_file.write("    if {[catch {mrd -size h $add} err]} {\n")                                       
            tcl_file.write("        return \"$add: $err\"\n")                                          
            tcl_file.write("    } else {\n")                                                           
            tcl_file.write("        return $err\n")                                                    
            tcl_file.write("    }\n")                                                                       
            tcl_file.write("}\n")
            tcl_file.write(tcl_content)
        os.chmod(f"{instance_name}.tcl", 0o775)
    print(f"tcl files generated sucessfully for {block_name} block")










##
# Copyright (c) 2021 Xilinx, Inc.
# 
# Permission is hereby granted, free of charge, to any person obtaining a copy of this software and 
# associated documentation files (the "Software"), to deal in the Software without restriction, including 
# without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, and/or sell 
# copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to 
# the following conditions: 
# 
# The above copyright notice and this permission notice shall be included in all copies or substantial 
# portions of the Software. 
# 
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT 
# LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, 
# WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE 
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
# 
# 
##
from pymongo import MongoClient
from api import settings
mongoClient = MongoClient(settings.MONGO_URI)
db = mongoClient.xregv.usercollection
def get_userdetails(uniquekey):
    global db
    return list(db.find({"uniqueid": uniquekey,"status":"A"},{"_id":0}))
    
def required_login(uniquekey):
    global db
    try:
        cursor = list(db.find({"uniqueid": uniquekey,"status":"A"}))
        if len(cursor):
            return True
        else:
            return False
    except:
        return False

def add_user(uniquekey,user):
    global db
    try:
        db.update_many({"uniqueid":uniquekey}, {"$set": user},upsert=True)
        return True
    except:
        return False

def get_user(*args):
    global db
    try:
        count = 0
        projectobj = {}
        uniquekey = ""
        for arg in args:
            if not count:
                uniquekey = arg
            else:
                projectobj[arg] = 1
            count =+1
        projectobj["_id"] = 0
        user = list(db.find({"uniqueid":uniquekey},projectobj))
        return user[0]
    except Exception as e:
        print("error--->>", str(e))
        return False

def set_user(uniquekey,setobj):
    print("function call")
    print("---------",uniquekey)
    for key in setobj:
        print(key)
    global db
    try:
        projectobj = {}
        for key in setobj:
            projectobj[key] = setobj[key]
        print("setobj------->>",projectobj)

        db.update_many({"uniqueid":uniquekey},{"$set": projectobj})
        return True
    except:
        return False

def del_user(uniquekey):
    global db
    try:
        db.update_many({"uniqueid": uniquekey},  {"$set": {"connectionstatus": 0, "status": "I"}})
        return True
    except:
        return False
